/*

	The Hacked Together Wave to Disk Configurator
    copyright (c) 1999 Eben Upton, Tom Lynn and Chris Lightfoot

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include <windows.h>
#include <shlobj.h>
#include <string.h>

HINSTANCE hinst;

struct
{
	DWORD minlength;
	char outputdirectory[MAX_PATH];
} configuration;

void SaveConfigurationToRegistry();

void SaveConfigurationToRegistry()
{
	DWORD dwdisp;
	HKEY key;
	int writeout = 0;

	if (RegCreateKeyEx(HKEY_LOCAL_MACHINE,
					   "Software\\Hacks 'R' Us\\Wave to Disk",
					   0, NULL, REG_OPTION_NON_VOLATILE, KEY_SET_VALUE,
					   NULL, &key, &dwdisp)
							== ERROR_SUCCESS)
	{
		if (RegSetValueEx(key, "OutputDirectory", 0, REG_SZ,
							(BYTE*)configuration.outputdirectory,
							strlen(configuration.outputdirectory))
							!= ERROR_SUCCESS)
		{
			MessageBox(0, "There was an error writing to the Registry.\n\nThis should probably not surprise you very much.", "HT Wave to Disk Configuration", 0);
		}

		if (RegSetValueEx(key, "MinLength", 0, REG_DWORD, (BYTE*)&(configuration.minlength), sizeof(DWORD))
							!= ERROR_SUCCESS)
		{
			MessageBox(0, "There was an error writing to the Registry.\n\nThis should probably not surprise you very much.", "HT Wave to Disk Configuration", 0);
		}

		RegCloseKey(key);
	}
}


void GetConfigurationFromRegistry();

void GetConfigurationFromRegistry()
{
	DWORD dwdisp;
	HKEY key;
	int writeout = 0;

	if (RegCreateKeyEx(HKEY_LOCAL_MACHINE,
					   "Software\\Hacks 'R' Us\\Wave to Disk",
					   0, NULL, REG_OPTION_NON_VOLATILE, KEY_QUERY_VALUE,
					   NULL, &key, &dwdisp)
							== ERROR_SUCCESS)
	{
		DWORD dwType, dwSize;
		char data[MAX_PATH];

		dwSize = MAX_PATH;
		if (RegQueryValueEx(key, "OutputDirectory", 0, &dwType, (BYTE*)data, &dwSize)
							== ERROR_SUCCESS)
		{
		   strncpy(configuration.outputdirectory, data, MAX_PATH);
		}
		else
		{
			writeout = 1;
			strncpy(configuration.outputdirectory, "C:\\", MAX_PATH);
		}

		dwSize = 256;
		if (RegQueryValueEx(key, "MinLength", 0, &dwType, (BYTE*)data, &dwSize)
							== ERROR_SUCCESS)
		{
			configuration.minlength = *((DWORD*)data);
		}
		else
		{
			writeout = 1;
			configuration.minlength = 262144;
		}

		RegCloseKey(key);
    }
	else
	{
		writeout = 1;
		strncpy(configuration.outputdirectory, "C:\\", MAX_PATH);
		configuration.minlength = 262144;
	}

	if (writeout)
		SaveConfigurationToRegistry();

}


int GetFolderName(char *fname, int n);

int GetFolderName(char *fname, int n)
{
	BROWSEINFO bi;
	ITEMIDLIST *iidl;
	char buffer[MAX_PATH];
	char realname[MAX_PATH];

	bi.hwndOwner = 0;
	bi.pidlRoot = NULL;
	bi.pszDisplayName = buffer;
	bi.lpszTitle = "Select the folder in which the Hacked-Together Wave to Disk driver will store its output files.";
	bi.ulFlags = BIF_RETURNONLYFSDIRS;
	bi.lpfn = NULL;
	bi.lParam = 0;
	bi.iImage = 0;

	iidl = SHBrowseForFolder(&bi);

	if (iidl)
	{
		SHGetPathFromIDList(iidl, realname);
		
		strncpy(fname, realname, n);
		return 1;
	}

	return 0;
}

BOOL WINAPI AboutDlgProc(HWND hwnd, UINT msg, WPARAM w, LPARAM l);

BOOL WINAPI AboutDlgProc(HWND hwnd, UINT msg, WPARAM w, LPARAM l)
{
	switch (msg)
	{
	case WM_INITDIALOG:
		return 1;

	case WM_COMMAND:
		if (LOWORD(w) == IDOK)
			EndDialog(hwnd, 0);

		return 1;

	default:
		return 0;
	}
}

#define EDIT_FOLDER	100
#define BTN_BROWSE	101
#define SLD_SIZE	102
#define BTN_ABOUT	103

BOOL WINAPI ConfigDlgProc(HWND hwnd, UINT msg, WPARAM w, LPARAM l);

BOOL WINAPI ConfigDlgProc(HWND hwnd, UINT msg, WPARAM w, LPARAM l)
{
	switch (msg)
	{
	case WM_INITDIALOG:
		SetFocus(GetDlgItem(hwnd, EDIT_FOLDER));

		// set up the silly slider
		SendMessage(GetDlgItem(hwnd, SLD_SIZE), TBM_SETRANGE, 1, MAKELONG(0, 1024));
		SendMessage(GetDlgItem(hwnd, SLD_SIZE), TBM_SETTICFREQ, 256, 0);
		SendMessage(GetDlgItem(hwnd, SLD_SIZE), TBM_SETPOS, 1, configuration.minlength / 1024);

		SetDlgItemText(hwnd, EDIT_FOLDER, configuration.outputdirectory);

		return 0;

	case WM_COMMAND:
		switch (LOWORD(w))
		{
		case IDOK:
			configuration.minlength = 1024 * SendMessage(GetDlgItem(hwnd, SLD_SIZE), TBM_GETPOS, 0, 0);
			GetDlgItemText(hwnd, EDIT_FOLDER, configuration.outputdirectory, MAX_PATH);

			if (!SetCurrentDirectory(configuration.outputdirectory))
			{
				if (MessageBox(hwnd, "The directory specified does not appear to exist. Create it?",
					"HT Wave to Disk Configuration", MB_YESNO) == IDYES)
				{
					if (!CreateDirectory(configuration.outputdirectory, NULL))
					{
						MessageBox(hwnd, "Could not create the directory. Please try again.", "HT Wave to Disk Configuration", 0);
						return 1;
					}
				}
				else
					return 0;

			}

			EndDialog(hwnd, 0);
			return 1;

		case IDCANCEL:
			EndDialog(hwnd, 1);
			return 1;

		case SLD_SIZE:
			return 1;

		case BTN_BROWSE:
			if (GetFolderName(configuration.outputdirectory, MAX_PATH))
				SetDlgItemText(hwnd, EDIT_FOLDER, configuration.outputdirectory);
			return 1;

		case BTN_ABOUT:
			DialogBox(hinst, MAKEINTRESOURCE(200), 0, (DLGPROC)AboutDlgProc);

			return 1;

		return 1;
		}

	default:
		return 0;

	}
}

int WINAPI WinMain(HINSTANCE hinst, HINSTANCE a, char *c, int b)
{
	a, b, c;

	::hinst = hinst;
	
	GetConfigurationFromRegistry();

	if (DialogBox(hinst, MAKEINTRESOURCE(100), 0, (DLGPROC)ConfigDlgProc))
	{
		return 1;
	}

	// save config
	SaveConfigurationToRegistry();	

	return 0;
}